#!/usr/bin/env python
from .schema import Schema
from .yamale_error import YamaleError


def make_schema(path=None, parser='PyYAML', validators=None, content=None):
    # validators = None means use default.
    # Import readers here so we can get version information in setup.py.
    from . import readers
    raw_schemas = readers.parse_yaml(path, parser, content=content)
    if not raw_schemas:
        raise ValueError('{} is an empty file!'.format(path))
    # First document is the base schema
    try:
        s = Schema(raw_schemas[0], path, validators=validators)
        # Additional documents contain Includes.
        for raw_schema in raw_schemas[1:]:
            s.add_include(raw_schema)
    except (TypeError, SyntaxError) as e:
        error = 'Schema error in file %s\n' % path
        error += str(e)
        raise SyntaxError(error)

    return s


def make_data(path=None, parser='PyYAML', content=None):
    from . import readers
    raw_data = readers.parse_yaml(path, parser, content=content)
    if len(raw_data) == 0:
        return [({}, path)]
    return [(d, path) for d in raw_data]


def validate(schema, data, strict=True, _raise_error=True):
    results = []
    is_valid = True
    for d, path in data:
        result = schema.validate(d, path, strict)
        results.append(result)
        is_valid = is_valid and result.isValid()
    if _raise_error and not is_valid:
        raise YamaleError(results)
    return results



import os

import pytest

from .. import command_line
from .. import yamale_error

dir_path = os.path.dirname(os.path.realpath(__file__))

parsers = ['pyyaml', 'PyYAML', 'ruamel']


@pytest.mark.parametrize('parser', parsers)
def test_bad_yaml(parser):
    with pytest.raises(ValueError) as e:
        command_line._router(
            'yamale/tests/command_line_fixtures/yamls/bad.yaml',
            'schema.yaml', 1, parser)
    assert "map.bad: '12.5' is not a str." in e.value.message


@pytest.mark.parametrize('parser', parsers)
def test_required_keys_yaml(parser):
    with pytest.raises(ValueError) as e:
        command_line._router(
            'yamale/tests/command_line_fixtures/yamls/required_keys_bad.yaml',
            'required_keys_schema.yaml', 1, parser)
    assert "map.key: Required field missing" in e.value.message


@pytest.mark.parametrize('parser', parsers)
def test_good_yaml(parser):
    command_line._router(
        'yamale/tests/command_line_fixtures/yamls/good.yaml',
        'schema.yaml', 1, parser)
    

@pytest.mark.parametrize('parser', parsers)
def test_good_relative_yaml(parser):
    command_line._router(
        'yamale/tests/command_line_fixtures/yamls/good.yaml',
        '../schema_dir/external.yaml', 1, parser)
    

@pytest.mark.parametrize('parser', parsers)
def test_external_glob_schema(parser):
    command_line._router(
        'yamale/tests/command_line_fixtures/yamls/good.yaml',
        os.path.join(dir_path, 'command_line_fixtures/schema_dir/ex*.yaml'), 1, parser)
    

def test_empty_schema_file():
    with pytest.raises(ValueError, match='is an empty file!'):
        command_line._router(
            'yamale/tests/command_line_fixtures/empty_schema/data.yaml',
            'empty_schema.yaml' , 1, 'PyYAML')


def test_external_schema():
    command_line._router(
        'yamale/tests/command_line_fixtures/yamls/good.yaml',
        os.path.join(dir_path, 'command_line_fixtures/schema_dir/external.yaml'), 1, 'PyYAML')


def test_bad_dir():
    with pytest.raises(ValueError):
        command_line._router(
            'yamale/tests/command_line_fixtures/yamls',
            'schema.yaml', 4, 'PyYAML')


def test_bad_strict():
    with pytest.raises(ValueError) as e:
        command_line._router(
            'yamale/tests/command_line_fixtures/yamls/required_keys_extra_element.yaml',
            'required_keys_schema.yaml',
            4, 'PyYAML', strict=True)
    assert "map.key2: Unexpected element" in e.value.message


def test_bad_issue_54():
    with pytest.raises(yamale_error.YamaleError) as e:
        command_line._router(
            'yamale/tests/fixtures/nested_issue_54.yaml',
            'nested.yaml',
            4, 'PyYAML', strict=True)
    assert 'string: Required field missing' in e.value.message
    assert 'number: Required field missing' in e.value.message
    assert 'integer: Required field missing' in e.value.message
    assert 'boolean: Required field missing' in e.value.message
    assert 'date: Required field missing' in e.value.message
    assert 'datetime: Required field missing' in e.value.message
    assert 'nest: Required field missing' in e.value.message
    assert 'list: Required field missing' in e.value.message

def test_nested_schema_issue_69():
    command_line._router('yamale/tests/command_line_fixtures/nestedYaml','schema.yaml', 1, 'PyYAML')




class Result(object):
    def __init__(self, errors):
        self.errors = errors

    def __str__(self):
        return '\n'.join(self.errors)

    def isValid(self):
        return len(self.errors) == 0


class ValidationResult(Result):
    def __init__(self, data, schema, errors):
        super(ValidationResult, self).__init__(errors)
        self.data = data
        self.schema = schema

    def __str__(self):
        if self.isValid():
            error_str = "'%s' is Valid" % self.data
        else:
            head_line_bits = ["Error validating data"]
            if self.data:
                head_line_bits.append("'{}'".format(self.data))
            if self.schema:
                head_line_bits.append("with schema '{}'".format(self.schema))
            head_line = ' '.join(head_line_bits)
            head_line += '\n\t'
            error_str = head_line + '\n\t'.join(self.errors)
        return error_str



import io
import pytest
import re
import yamale

from . import get_fixture
from .. import validators as val

types = {
    'schema': 'types.yaml',
    'bad': 'types_bad_data.yaml',
    'good': 'types_good_data.yaml'
}

nested = {
    'schema': 'nested.yaml',
    'bad': 'nested_bad_data.yaml',
    'good': 'nested_good_data.yaml'
}

custom = {
    'schema': 'custom_types.yaml',
    'bad': 'custom_types_bad.yaml',
    'good': 'custom_types_good.yaml'
}

keywords = {
    'schema': 'keywords.yaml',
    'bad': 'keywords_bad.yaml',
    'good': 'keywords_good.yaml'
}

lists = {
    'schema': 'lists.yaml',
    'bad': 'lists_bad.yaml',
    'bad2': 'lists_bad2.yaml',
    'good': 'lists_good.yaml'
}

maps = {
    'schema': 'map.yaml',
    'bad': 'map_bad.yaml',
    'bad2': 'map_bad2.yaml',
    'good': 'map_good.yaml'
}

anys = {
    'schema': 'any.yaml',
    'bad': 'any_bad.yaml',
    'good': 'any_good.yaml'
}

list_include = {
    'schema': 'list_include.yaml',
    'good': 'list_include_good.yaml'
}

issue_22 = {
    'schema': 'issue_22.yaml',
    'good': 'issue_22_good.yaml'
}

issue_50 = {
    'schema': 'issue_50.yaml',
    'good': 'issue_50_good.yaml'
}

regexes = {
    'schema': 'regex.yaml',
    'bad': 'regex_bad.yaml',
    'good': 'regex_good.yaml'
}

ips = {
    'schema': 'ip.yaml',
    'bad': 'ip_bad.yaml',
    'good': 'ip_good.yaml'
}

macs = {
    'schema': 'mac.yaml',
    'bad': 'mac_bad.yaml',
    'good': 'mac_good.yaml'
}

nested_map = {
    'schema': 'nested_map.yaml',
    'good': 'nested_map_good.yaml'
}

top_level_map = {
    'schema': 'top_level_map.yaml',
    'good': 'top_level_map_good.yaml'
}

include_validator = {
    'schema': 'include_validator.yaml',
    'good': 'include_validator_good.yaml',
    'bad': 'include_validator_bad.yaml'
}

strict_map = {
    'schema': 'strict_map.yaml',
    'good': 'strict_map_good.yaml',
    'bad': 'strict_map_bad.yaml'
}

mixed_strict_map = {
    'schema': 'mixed_strict_map.yaml',
    'good': 'mixed_strict_map_good.yaml',
    'bad': 'mixed_strict_map_bad.yaml'
}

strict_list = {
    'schema': 'strict_list.yaml',
    'good': 'strict_list_good.yaml',
    'bad': 'strict_list_bad.yaml'
}

nested_map2 = {
    'schema': 'nested_map2.yaml',
    'good': 'nested_map2_good.yaml',
    'bad': 'nested_map2_bad.yaml'
}

static_list = {
    'schema': 'static_list.yaml',
    'good': 'static_list_good.yaml',
    'bad': 'static_list_bad.yaml'
}

nested_issue_54 = {
    'schema': 'nested.yaml',
    'bad': 'nested_issue_54.yaml',
    'good': 'nested_good_data.yaml'
}

map_key_constraint = {
    'schema': 'map_key_constraint.yaml',
    'good': 'map_key_constraint_good.yaml',
    'bad_base': 'map_key_constraint_bad_base.yaml',
    'bad_nest': 'map_key_constraint_bad_nest.yaml',
    'bad_nest_con': 'map_key_constraint_bad_nest_con.yaml',
}

numeric_bool_coercion = {
    'schema': 'numeric_bool_coercion.yaml',
    'good': 'numeric_bool_coercion_good.yaml',
    'bad': 'numeric_bool_coercion_bad.yaml',
}

subset = {
    'schema': 'subset.yaml',
    'good': 'subset_good.yaml',
    'good2': 'subset_good2.yaml',
    'bad': 'subset_bad.yaml',
    'bad2': 'subset_bad2.yaml',
    'bad3': 'subset_bad3.yaml'
}

subset_empty = {
    'schema': 'subset_empty.yaml',
    'good': 'subset_empty_good.yaml',
    'good2': 'subset_empty_good2.yaml'
}

subset_nodef = {
    'schema': 'subset_nodef.yaml'
}

test_data = [
    types, nested, custom,
    keywords, lists, maps,
    anys, list_include, issue_22,
    issue_50, regexes, ips, macs,
    nested_map, top_level_map,
    include_validator, strict_map,
    mixed_strict_map, strict_list,
    nested_map2, static_list,
    nested_issue_54,
    map_key_constraint,
    numeric_bool_coercion,
    subset, subset_empty
]

for d in test_data:
    for key in d.keys():
        if key == 'schema':
            d[key] = yamale.make_schema(get_fixture(d[key]))
        else:
            d[key] = yamale.make_data(get_fixture(d[key]))


def test_tests():
    """ Make sure the test runner is working."""
    assert 1 + 1 == 2


def test_flat_make_schema():
    assert isinstance(types['schema']._schema['string'], val.String)


def test_nested_schema():
    nested_schema = nested['schema']._schema
    assert isinstance(nested_schema['string'], val.String)
    assert isinstance(nested_schema['list'], (list, tuple))
    assert isinstance(nested_schema['list'][0], val.String)


@pytest.mark.parametrize('data_map', test_data)
def test_good(data_map):
    for k, v in data_map.items():
        if k.startswith('good'):
            yamale.validate(data_map['schema'], data_map[k])


def test_bad_validate():
    assert count_exception_lines(types['schema'], types['bad']) == 9


def test_bad_nested():
    assert count_exception_lines(nested['schema'], nested['bad']) == 2


def test_bad_nested_issue_54():
    exp = [
        'string: Required field missing',
        'number: Required field missing',
        'integer: Required field missing',
        'boolean: Required field missing',
        'date: Required field missing',
        'datetime: Required field missing',
        'nest: Required field missing',
        'list: Required field missing'
    ]
    match_exception_lines(nested_issue_54['schema'], nested_issue_54['bad'], exp)

def test_bad_custom():
    assert count_exception_lines(custom['schema'], custom['bad']) == 1


def test_bad_lists():
    assert count_exception_lines(lists['schema'], lists['bad']) == 6


def test_bad2_lists():
    assert count_exception_lines(lists['schema'], lists['bad2']) == 2


def test_bad_maps():
    assert count_exception_lines(maps['schema'], maps['bad']) == 7

def test_bad_maps2():
    assert count_exception_lines(maps['schema'], maps['bad2']) == 1

def test_bad_keywords():
    assert count_exception_lines(keywords['schema'], keywords['bad']) == 9


def test_bad_anys():
    assert count_exception_lines(anys['schema'], anys['bad']) == 5


def test_bad_regexes():
    assert count_exception_lines(regexes['schema'], regexes['bad']) == 4


def test_bad_include_validator():
    exp = ["key1: 'a_string' is not a int."]
    match_exception_lines(include_validator['schema'],
                          include_validator['bad'],
                          exp)


def test_bad_schema():
    with pytest.raises(SyntaxError) as excinfo:
        yamale.make_schema(get_fixture('bad_schema.yaml'))
    assert 'fixtures/bad_schema.yaml' in str(excinfo.value)


def test_empty_schema():
    with pytest.raises(ValueError) as excinfo:
        yamale.make_schema(get_fixture('empty_schema.yaml'))
    assert 'empty_schema.yaml is an empty file!' in str(excinfo.value)


@pytest.mark.parametrize(
    "schema_filename",
    ['bad_schema_rce.yaml', 'bad_schema_rce2.yaml', 'bad_schema_rce3.yaml', 'bad_schema_rce4.yaml']
)
def test_vulnerable_schema(schema_filename):
    with pytest.raises(SyntaxError) as excinfo:
        yamale.make_schema(get_fixture(schema_filename))
    assert schema_filename in str(excinfo.value)


def test_list_is_not_a_map():
    exp = [" : '[1, 2]' is not a map"]
    match_exception_lines(strict_map['schema'],
                          strict_list['good'],
                          exp)


def test_bad_strict_map():
    exp = ['extra: Unexpected element']
    match_exception_lines(strict_map['schema'],
                          strict_map['bad'],
                          exp,
                          strict=True)


def test_bad_strict_list():
    exp = ['2: Unexpected element']
    match_exception_lines(strict_list['schema'],
                          strict_list['bad'],
                          exp,
                          strict=True)


def test_bad_mixed_strict_map():
    exp = ['field3.extra: Unexpected element']
    match_exception_lines(mixed_strict_map['schema'],
                          mixed_strict_map['bad'],
                          exp)


def test_bad_nested_map2():
    exp = ['field1.field1_1: Required field missing']
    match_exception_lines(nested_map2['schema'],
                          nested_map2['bad'],
                          exp)


def test_bad_static_list():
    exp = ['0: Required field missing']
    match_exception_lines(static_list['schema'],
                          static_list['bad'],
                          exp)


def test_bad_map_key_constraint_base():
    exp = [": Key error - 'bad' is not a int."]
    match_exception_lines(map_key_constraint['schema'],
                          map_key_constraint['bad_base'],
                          exp)


def test_bad_map_key_constraint_nest():
    exp = ["1.0: Key error - '100' is not a str."]
    match_exception_lines(map_key_constraint['schema'],
                          map_key_constraint['bad_nest'],
                          exp)


def test_bad_map_key_constraint_nest_con():
    exp = [
        "1.0: Key error - '100' is not a str.",
        "1.0: Key error - 'baz' contains excluded character 'z'",
    ]
    match_exception_lines(map_key_constraint['schema'],
                          map_key_constraint['bad_nest_con'],
                          exp)


def test_bad_numeric_bool_coercion():
    exp = [
        "integers.0: 'False' is not a int.",
        "integers.1: 'True' is not a int.",
        "numbers.0: 'False' is not a num.",
        "numbers.1: 'True' is not a num.",
    ]
    match_exception_lines(numeric_bool_coercion['schema'],
                          numeric_bool_coercion['bad'],
                          exp)

def test_bad_subset():
    exp = [
        "subset_list: 'subset' may not be an empty set."
    ]
    match_exception_lines(subset['schema'],
                          subset['bad'],
                          exp)

def test_bad_subset2():
    exp = [
        "subset_list: '[1]' is not a int.",
        "subset_list: '[1]' is not a str."
    ]
    match_exception_lines(subset['schema'],
                          subset['bad2'],
                          exp)

def test_bad_subset3():
    exp = [
        "subset_list: '{'a': 1}' is not a int.",
        "subset_list: '{'a': 1}' is not a str."
    ]
    match_exception_lines(subset['schema'],
                          subset['bad3'],
                          exp)

def test_nodef_subset_schema():
    with pytest.raises(ValueError) as e:
        yamale.make_schema(get_fixture(subset_nodef['schema']))

    assert "'subset' requires at least one validator!" in str(e.value)

@pytest.mark.parametrize("use_schema_string,use_data_string,expected_message_re", [
    (False, False, "^Error validating data '.*?' with schema '.*?'\n\t"),
    (True, False, "^Error validating data '.*?'\n\t"),
    (False, True, "^Error validating data with schema '.*?'\n\t"),
    (True, True, "^Error validating data\n\t"),
])
def test_validate_errors(use_schema_string, use_data_string, expected_message_re):
    schema_path = get_fixture('types.yaml')
    data_path = get_fixture('types_bad_data.yaml')
    if use_schema_string:
        with io.open(schema_path, encoding='utf-8') as f:
            schema = yamale.make_schema(content=f.read())
    else:
        schema = yamale.make_schema(schema_path)
    if use_data_string:
        with io.open(data_path, encoding='utf-8') as f:
            data = yamale.make_data(content=f.read())
    else:
        data = yamale.make_data(data_path)
    with pytest.raises(yamale.yamale_error.YamaleError) as excinfo:
        yamale.validate(schema, data)
    assert re.match(expected_message_re, excinfo.value.message, re.MULTILINE), \
        'Message {} should match {}'.format(
            excinfo.value.message, expected_message_re
        )


def match_exception_lines(schema, data, expected, strict=False):
    with pytest.raises(ValueError) as e:
        yamale.validate(schema, data, strict)

    got = e.value.results[0].errors
    got.sort()
    expected.sort()
    assert got == expected


def count_exception_lines(schema, data, strict=False):
    with pytest.raises(ValueError) as e:
        yamale.validate(schema, data, strict)
    result = e.value.results[0]
    return len(result.errors)



import io
import pytest
from .. import yaml_reader
from yamale.tests import get_fixture

parsers = ['pyyaml', 'PyYAML', 'ruamel']
TYPES = get_fixture('types.yaml')
NESTED = get_fixture('nested.yaml')
KEYWORDS = get_fixture('keywords.yaml')


@pytest.mark.parametrize('parser', parsers)
@pytest.mark.parametrize('use_string', [True, False])
def test_parse(parser, use_string):
    if use_string:
        with io.open(TYPES, encoding='utf-8') as f:
            content = f.read()
        a = yaml_reader.parse_yaml(parser=parser, content=content)[0]
    else:
        a = yaml_reader.parse_yaml(TYPES, parser)[0]
    assert a['string'] == 'str()'


def test_parse_validates_arguments():
    with pytest.raises(TypeError):
        yaml_reader.parse_yaml(path=TYPES, content="name: Bob")
    with pytest.raises(TypeError):
        yaml_reader.parse_yaml(path=None, content=None)


@pytest.mark.parametrize('parser', parsers)
def test_types(parser):
    t = yaml_reader.parse_yaml(TYPES, parser)[0]
    assert t['string'] == 'str()'
    assert t['number'] == 'num()'
    assert t['boolean'] == 'bool()'
    assert t['integer'] == 'int()'


@pytest.mark.parametrize('parser', parsers)
def test_keywords(parser):
    t = yaml_reader.parse_yaml(KEYWORDS, parser)[0]
    assert t['optional_min'] == 'int(min=1, required=False)'


@pytest.mark.parametrize('parser', parsers)
def test_nested(parser):
    t = yaml_reader.parse_yaml(NESTED, parser)[0]
    assert t['list'][-1]['string'] == 'str()'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    Validate yaml files and check them against their schemas. Designed to be used outside of Vagrant.

    Just install Yamale:
        pip install yamale
"""

import argparse
import glob
import os
from multiprocessing import Pool
from .yamale_error import YamaleError
from .schema.validationresults import Result
from .version import __version__

import yamale

schemas = {}


def _validate(schema_path, data_path, parser, strict, _raise_error):
    schema = schemas.get(schema_path)
    try:
        if not schema:
            schema = yamale.make_schema(schema_path, parser)
            schemas[schema_path] = schema
    except (SyntaxError, ValueError) as e:
        results = [Result([str(e)])]
        if not _raise_error:
            return results
        raise YamaleError(results)
    data = yamale.make_data(data_path, parser)
    return yamale.validate(schema, data, strict, _raise_error)


def _find_data_path_schema(data_path, schema_name):
    """ Starts in the data file folder and recursively looks
    in parents for `schema_name` """
    if not data_path or data_path == os.path.abspath(os.sep) or data_path == '.':
        return None
    directory = os.path.dirname(data_path)
    path = glob.glob(os.path.join(directory, schema_name))
    if not path:
        return _find_schema(directory, schema_name)
    return path[0]


def _find_schema(data_path, schema_name):
    """ Checks if `schema_name` is a valid file, if not
    searches in `data_path` for it. """

    directory = os.path.dirname(data_path)
    path = glob.glob(os.path.join(directory, schema_name))
    for p in path:
        if os.path.isfile(p):
            return p

    return _find_data_path_schema(data_path, schema_name)


def _validate_single(yaml_path, schema_name, parser, strict):
    print('Validating %s...' % yaml_path)
    s = _find_schema(yaml_path, schema_name)
    if not s:
        raise ValueError("Invalid schema name for '{}' or schema not found.".format(schema_name))
    _validate(s, yaml_path, parser, strict, True)


def _validate_dir(root, schema_name, cpus, parser, strict):
    pool = Pool(processes=cpus)
    res = []
    error_messages = []
    print('Finding yaml files...')
    for root, dirs, files in os.walk(root):
        for f in files:
            if (f.endswith('.yaml') or f.endswith('.yml')) and f != schema_name:
                d = os.path.join(root, f)
                s = _find_schema(d, schema_name)
                if s:
                    res.append(pool.apply_async(_validate,
                                                (s, d, parser, strict, False)))
                else:
                    print('No schema found for: %s' % d)

    print('Found %s yaml files.' % len(res))
    print('Validating...')
    for r in res:
        sub_results = r.get(timeout=300)
        error_messages.extend([str(sub_result)
                               for sub_result in sub_results
                               if not sub_result.isValid()])
    pool.close()
    pool.join()
    if error_messages:
        raise ValueError('\n----\n'.join(set(error_messages)))


def _router(root, schema_name, cpus, parser, strict=True):
    root = os.path.abspath(root)
    if os.path.isfile(root):
        _validate_single(root, schema_name, parser, strict)
    else:
        _validate_dir(root, schema_name, cpus, parser, strict)


def main():
    parser = argparse.ArgumentParser(description='Validate yaml files.')
    parser.add_argument('path', metavar='PATH', default='./', nargs='?',
                        help='folder to validate. Default is current directory.')
    parser.add_argument('-V', '--version', action='version', version=__version__)
    parser.add_argument('-s', '--schema', default='schema.yaml',
                        help='filename of schema. Default is schema.yaml.')
    parser.add_argument('-n', '--cpu-num', default=4, type=int,
                        help='number of CPUs to use. Default is 4.')
    parser.add_argument('-p', '--parser', default='pyyaml',
                        help='YAML library to load files. Choices are "ruamel" or "pyyaml" (default).')
    parser.add_argument('--no-strict', action='store_true',
                        help='Disable strict mode, unexpected elements in the data will be accepted.')
    args = parser.parse_args()
    try:
        _router(args.path, args.schema, args.cpu_num, args.parser, not args.no_strict)
    except (SyntaxError, NameError, TypeError, ValueError) as e:
        print('Validation failed!\n%s' % str(e))
        exit(1)
    try:
        print('Validation success! 👍')
    except UnicodeEncodeError:
        print('Validation success!')


if __name__ == '__main__':
    main()



from datetime import date, datetime
from yamale import validators as val


def test_validator_defaults():
    """
    Unit test the dictionary of default validators.
    """
    assert val.DefaultValidators[val.String.tag] is val.String
    assert val.DefaultValidators[val.Any.__name__] is val.Any


def test_equality():
    assert val.String() == val.String()
    assert val.String(hello='wat') == val.String(hello='wat')
    assert val.String(hello='wat') != val.String(hello='nope')
    assert val.Boolean('yep') != val.Boolean('nope')


def test_integer():
    v = val.Integer()
    assert v.is_valid(1)
    assert not v.is_valid('1')
    assert not v.is_valid(1.34)


def test_string():
    v = val.String()
    assert v.is_valid('1')
    assert not v.is_valid(1)


def test_regex():
    v = val.Regex(r'^(abc)\1?de$', name='test regex')
    assert v.is_valid('abcabcde')
    assert not v.is_valid('abcabcabcde')
    assert not v.is_valid('\12')
    assert v.fail('woopz') == '\'woopz\' is not a test regex.'

    v = val.Regex(r'[a-z0-9]{3,}s\s$', ignore_case=True)
    assert v.is_valid('b33S\v')
    assert v.is_valid('B33s\t')
    assert not v.is_valid(' b33s ')
    assert not v.is_valid('b33s  ')
    assert v.fail('fdsa') == '\'fdsa\' is not a regex match.'

    v = val.Regex(r'A.+\d$', ignore_case=False, multiline=True)
    assert v.is_valid('A_-3\n\n')
    assert not v.is_valid('a!!!!!5\n\n')

    v = val.Regex(r'.*^Ye.*s\.', ignore_case=True, multiline=True, dotall=True)
    assert v.is_valid('YEeeEEEEeeeeS.')
    assert v.is_valid('What?\nYes!\nBEES.\nOK.')
    assert not v.is_valid('YES-TA-TOES?')
    assert not v.is_valid('\n\nYaes.')


def test_number():
    v = val.Number()
    assert v.is_valid(1)
    assert v.is_valid(1.3425235)
    assert not v.is_valid('str')


def test_boolean():
    v = val.Boolean()
    assert v.is_valid(True)
    assert v.is_valid(False)
    assert not v.is_valid('')
    assert not v.is_valid(0)


def test_date():
    v = val.Day()
    assert v.is_valid(date(2015, 1, 1))
    assert v.is_valid(datetime(2015, 1, 1, 1))
    assert not v.is_valid('')
    assert not v.is_valid(0)


def test_datetime():
    v = val.Timestamp()
    assert v.is_valid(datetime(2015, 1, 1, 1))
    assert not v.is_valid(date(2015, 1, 1))
    assert not v.is_valid('')
    assert not v.is_valid(0)


def test_list():
    v = val.List()
    assert v.is_valid([])
    assert v.is_valid(())
    assert not v.is_valid('')
    assert not v.is_valid(0)


def test_null():
    v = val.Null()
    assert v.is_valid(None)
    assert not v.is_valid('None')
    assert not v.is_valid(0)
    assert not v.is_valid(float('nan'))
    assert not v.is_valid({})

def test_ip():
    v = val.Ip()
    assert v.is_valid('192.168.1.1')
    assert v.is_valid('192.168.1.255')
    assert v.is_valid('192.168.3.1/24')
    assert v.is_valid('2001:db8::')
    assert v.is_valid('2001:db8::/64')
    assert not v.is_valid('257.192.168.1')
    assert not v.is_valid('192.168.1.256')
    assert not v.is_valid('2001:db8::/129')
    assert not v.is_valid('2001:dg8::/127')
    assert not v.is_valid('asdf')

def test_mac():
    v = val.Mac()
    assert v.is_valid('12:34:56:78:90:ab')
    assert v.is_valid('1234:5678:90ab')
    assert v.is_valid('12-34-56-78-90-ab')
    assert v.is_valid('1234-5678-90ab')

    assert v.is_valid('12:34:56:78:90:AB')
    assert v.is_valid('1234:5678:90AB')
    assert v.is_valid('12-34-56-78-90-AB')
    assert v.is_valid('1234-5678-90AB')

    assert v.is_valid('ab:cd:ef:12:34:56')
    assert v.is_valid('abcd:ef12:3456')
    assert v.is_valid('ab-cd-ef-12-34-56')
    assert v.is_valid('abcd-ef12-3456')

    assert v.is_valid('AB:CD:EF:12:34:56')
    assert v.is_valid('ABCD:EF12:3456')
    assert v.is_valid('AB-CD-EF-12-34-56')
    assert v.is_valid('ABCD-EF12-3456')

    assert not v.is_valid('qwertyuiop')
    assert not v.is_valid('qw-er-ty-12-34-56')
    assert not v.is_valid('ab:cd:ef:12:34:56:78')
    assert not v.is_valid('abcdefghijkl')
    assert not v.is_valid('1234567890ax')



import glob
import os
import itertools

from unittest import TestCase

import yamale


class YamaleTestCase(TestCase):
    """ TestCase for easily validating YAML in your own tests.
    `schema`: String of path to the schema file to use. One schema file per test case.
    `yaml`: String or list of yaml files to validate. Accepts globs.
    `base_dir`: String path to prepend to all other paths. This is optional.
    """

    schema = None
    yaml = None
    base_dir = None

    def validate(self, validators=None):
        schema = self.schema
        yaml = self.yaml
        base_dir = self.base_dir

        if schema is None:
            return

        if type(yaml) != list:
            yaml = [yaml]

        if base_dir is not None:
            schema = os.path.join(base_dir, schema)
            yaml = {os.path.join(base_dir, y) for y in yaml}

        # Run yaml through glob and flatten list
        yaml = set(itertools.chain(*map(glob.glob, yaml)))

        # Remove schema from set of data files
        yaml = yaml - {schema}

        yamale_schema = yamale.make_schema(schema, validators=validators)
        yamale_data = itertools.chain(*map(yamale.make_data, yaml))

        for result in yamale.validate(yamale_schema, yamale_data):
            if not result.isValid():
                raise ValueError(result)
        return True




import ast

from .. import validators as val

safe_globals = ('True', 'False', 'None')
safe_builtins = dict((f, __builtins__[f]) for f in safe_globals)


def _validate_expr(call_node, validators):
    # Validate that the expression uses a known, registered validator.
    try:
        func_name = call_node.func.id
    except AttributeError:
        raise SyntaxError('Schema expressions must be enclosed by a validator.')
    if func_name not in validators:
        raise SyntaxError('Not a registered validator: \'%s\'. ' % func_name)
    # Validate that all args are constant literals, validator names,  or other call nodes
    arg_values = call_node.args + [kw.value for kw in call_node.keywords]
    for arg in arg_values:
        base_arg = arg.operand if isinstance(arg, ast.UnaryOp) else arg
        if isinstance(base_arg, ast.Constant):
            continue
        elif isinstance(base_arg, ast.Name) and base_arg.id in validators:
            continue
        elif isinstance(base_arg, ast.Call):
            _validate_expr(base_arg, validators)
        else:
            raise SyntaxError(
                'Argument values must either be constant literals, or else '
                'reference other validators.')


def parse(validator_string, validators=None):
    validators = validators or val.DefaultValidators
    try:
        tree = ast.parse(validator_string, mode='eval')
        _validate_expr(tree.body, validators)
        # evaluate with access to a limited global scope only
        return eval(compile(tree, '<ast>', 'eval'),
                    {'__builtins__': safe_builtins},
                    validators)
    except (SyntaxError, NameError, TypeError) as e:
        raise SyntaxError(
            'Invalid schema expression: \'%s\'. ' % validator_string +
            str(e)
        )



import datetime
from yamale import validators as val


def test_length_min():
    v = val.String(min=2)
    assert v.is_valid('abcd')
    assert v.is_valid('ab')
    assert not v.is_valid('a')


def test_length_max():
    v = val.String(max=3)
    assert v.is_valid('abc')
    assert v.is_valid('ab')
    assert not v.is_valid('abcd')


def test_number_max():
    v = val.Number(min=.5)
    assert v.is_valid(4)
    assert v.is_valid(.5)
    assert not v.is_valid(.1)


def test_number_min():
    v = val.Integer(max=10)
    assert v.is_valid(4)
    assert v.is_valid(10)
    assert not v.is_valid(11)


def test_timestamp_min():
    v = val.Timestamp(min=datetime.datetime(2010, 1, 1))
    assert v.is_valid(datetime.datetime(2010, 1, 1))
    assert v.is_valid(datetime.datetime(2011, 2, 2))
    assert not v.is_valid(datetime.datetime(2009, 12, 31))


def test_timestamp_max():
    v = val.Timestamp(max=datetime.datetime(2010, 1, 1))
    assert v.is_valid(datetime.datetime(2010, 1, 1))
    assert v.is_valid(datetime.datetime(2009, 2, 2))
    assert not v.is_valid(datetime.datetime(2010, 2, 2))


def test_day_min():
    v = val.Day(min=datetime.date(2010, 1, 1))
    assert v.is_valid(datetime.date(2010, 1, 1))
    assert v.is_valid(datetime.date(2011, 2, 2))
    assert not v.is_valid(datetime.date(2009, 12, 31))


def test_day_max():
    v = val.Day(max=datetime.date(2010, 1, 1))
    assert v.is_valid(datetime.date(2010, 1, 1))
    assert v.is_valid(datetime.date(2009, 2, 2))
    assert not v.is_valid(datetime.date(2010, 2, 2))


def test_str_equals():
    v = val.String(equals='abcd')
    assert v.is_valid('abcd')
    assert not v.is_valid('abcde')
    assert not v.is_valid('c')


def test_str_equals_ignore_case():
    v = val.String(equals='abcd', ignore_case=True)
    assert v.is_valid('abCd')
    assert not v.is_valid('abcde')
    assert not v.is_valid('C')


def test_str_starts_with():
    v = val.String(starts_with='abc')
    assert v.is_valid('abcd')
    assert not v.is_valid('bcd')
    assert not v.is_valid('c')


def test_str_starts_with_ignore_case():
    v = val.String(starts_with='abC', ignore_case=True)
    assert v.is_valid('abCde')
    assert v.is_valid('abcde')
    assert not v.is_valid('bcd')
    assert not v.is_valid('C')


def test_str_ends_with():
    v = val.String(ends_with='abcd')
    assert v.is_valid('abcd')
    assert not v.is_valid('abcde')
    assert not v.is_valid('c')


def test_str_ends_with_ignore_case():
    v = val.String(ends_with='abC', ignore_case=True)
    assert v.is_valid('xyzabC')
    assert v.is_valid('xyzabc')
    assert not v.is_valid('cde')
    assert not v.is_valid('C')


def test_str_matches():
    v = val.String(matches=r'^(abc)\1?de$')
    assert v.is_valid('abcabcde')
    assert not v.is_valid('abcabcabcde')
    assert not v.is_valid('\12')

    v = val.String(matches=r'[a-z0-9]{3,}s\s$', ignore_case=True)
    assert v.is_valid('b33S\v')
    assert v.is_valid('B33s\t')
    assert not v.is_valid(' b33s ')
    assert not v.is_valid('b33s  ')

    v = val.String(matches=r'A.+\d$', ignore_case=False, multiline=True)
    assert v.is_valid('A_-3\n\n')
    assert not v.is_valid('a!!!!!5\n\n')

    v = val.String(matches=r'.*^Ye.*s\.', ignore_case=True, multiline=True, dotall=True)
    assert v.is_valid('YEeeEEEEeeeeS.')
    assert v.is_valid('What?\nYes!\nBEES.\nOK.')
    assert not v.is_valid('YES-TA-TOES?')
    assert not v.is_valid('\n\nYaes.')


def test_char_exclude():
    v = val.String(exclude='abcd')
    assert v.is_valid('efg')
    assert not v.is_valid('abc')
    assert not v.is_valid('c')


def test_char_exclude_igonre_case():
    v = val.String(exclude='abcd', ignore_case=True)
    assert v.is_valid('efg')
    assert v.is_valid('Efg')
    assert not v.is_valid('abc')
    assert not v.is_valid('Def')
    assert not v.is_valid('c')


def test_ip4():
    v = val.Ip(version=4)
    assert v.is_valid('192.168.1.1')
    assert v.is_valid('192.168.1.255')
    assert v.is_valid('192.168.3.1/24')
    assert not v.is_valid('2001:db8::')
    assert not v.is_valid('2001:db8::/64')

def test_ip6():
    v = val.Ip(version=6)
    assert not v.is_valid('192.168.1.1')
    assert not v.is_valid('192.168.1.255')
    assert not v.is_valid('192.168.3.1/24')
    assert v.is_valid('2001:db8::')
    assert v.is_valid('2001:db8::/64')



import pytest
from pytest import raises
from .. import parse_yaml

parsers = ['pyyaml', 'PyYAML', 'ruamel']


@pytest.mark.parametrize('parser', parsers)
def test_reader_error(parser):
    with raises(IOError):
        parse_yaml('wat', parser)



from __future__ import absolute_import
import re
import datetime
import ipaddress

from yamale.util import to_unicode
from .base import Validator
from .. import util


class Constraint(object):
    keywords = {}  # Keywords and types accepted by this constraint
    is_active = False

    def __init__(self, value_type, kwargs):
        self._parseKwargs(kwargs)

    def _parseKwargs(self, kwargs):
        for kwarg, kwtype in self.keywords.items():
            value = self.get_kwarg(kwargs, kwarg, kwtype)
            setattr(self, kwarg, value)

    def get_kwarg(self, kwargs, key, kwtype):
        try:
            value = kwargs[key]
        except KeyError:
            return None

        # Activate this constraint
        self.is_active = True

        if isinstance(value, kwtype):
            # value already correct type, return
            return value

        try:  # Try to convert value
            # Is this value one of the datetime types?
            if kwtype == datetime.date:
                time = datetime.datetime.strptime(value, '%Y-%m-%d')
                return datetime.date(time.year, time.month, time.day)

            if kwtype == datetime.datetime:
                return datetime.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')

            return kwtype(value)
        except (TypeError, ValueError):
            raise SyntaxError('%s is not a %s' % (key, kwtype))

    def is_valid(self, value):
        if not self.is_active:
            return None

        if not self._is_valid(value):
            return self._fail(value)

        return None

    def _fail(self, value):
        return '\'%s\' violates %s.' % (value, self.__class__.__name__)


class Min(Constraint):
    fail = '%s is less than %s'

    def __init__(self, value_type, kwargs):
        self.keywords = {'min': value_type}
        super(Min, self).__init__(value_type, kwargs)

    def _is_valid(self, value):
        return self.min <= value

    def _fail(self, value):
        return self.fail % (value, self.min)


class Max(Constraint):
    fail = '%s is greater than %s'

    def __init__(self, value_type, kwargs):
        self.keywords = {'max': value_type}
        super(Max, self).__init__(value_type, kwargs)

    def _is_valid(self, value):
        return self.max >= value

    def _fail(self, value):
        return self.fail % (value, self.max)


class LengthMin(Constraint):
    keywords = {'min': int}
    fail = 'Length of %s is less than %s'

    def _is_valid(self, value):
        return self.min <= len(value)

    def _fail(self, value):
        return self.fail % (value, self.min)


class LengthMax(Constraint):
    keywords = {'max': int}
    fail = 'Length of %s is greater than %s'

    def _is_valid(self, value):
        return self.max >= len(value)

    def _fail(self, value):
        return self.fail % (value, self.max)


class Key(Constraint):
    keywords = {'key': Validator}
    fail = 'Key error - %s'

    def _is_valid(self, value):
        for k in value.keys():
            if self.key.validate(k) != []:
                return False
        return True

    def _fail(self, value):
        error_list = []
        for k in value.keys():
            error_list.extend(self.key.validate(k))
        return [self.fail % (e) for e in error_list]


class StringEquals(Constraint):
    keywords = {'equals': str, 'ignore_case': bool}
    fail = '%s does not equal %s'

    def _is_valid(self, value):
        # Check if the function has only been called due to ignore_case
        if self.equals is not None:
            if self.ignore_case is not None:
                if not self.ignore_case:
                    return value == self.equals
                else:
                    return value.casefold() == self.equals.casefold()
            else:
                return value == self.equals
        else:
            return True

    def _fail(self, value):
        return self.fail % (value, self.equals)


class StringStartsWith(Constraint):
    keywords = {'starts_with': str, 'ignore_case': bool}
    fail = '%s does not start with %s'

    def _is_valid(self, value):
        # Check if the function has only been called due to ignore_case
        if self.starts_with is not None:
            if self.ignore_case is not None:
                if not self.ignore_case:
                    return value.startswith(self.starts_with)
                else:
                    length = len(self.starts_with)
                    if length <= len(value):
                        return value[:length].casefold() == self.starts_with.casefold()
                    else:
                        return False
            else:
                return value.startswith(self.starts_with)
        else:
            return True

    def _fail(self, value):
        return self.fail % (value, self.starts_with)


class StringEndsWith(Constraint):
    keywords = {'ends_with': str, 'ignore_case': bool}
    fail = '%s does not end with %s'

    def _is_valid(self, value):
        # Check if the function has only been called due to ignore_case
        if self.ends_with is not None:
            if self.ignore_case is not None:
                if not self.ignore_case:
                    return value.endswith(self.ends_with)
                else:
                    length = len(self.ends_with)
                    if length <= len(value):
                        return value[-length:].casefold() == self.ends_with.casefold()
                    else:
                        return False
            else:
                return value.endswith(self.ends_with)
        else:
            return True

    def _fail(self, value):
        return self.fail % (value, self.ends_with)


class StringMatches(Constraint):
    keywords = {'matches': str}
    fail = '%s is not a regex match.'

    _regex_flags = {'ignore_case': re.I, 'multiline': re.M, 'dotall': re.S}

    def __init__(self, value_type, kwargs):
        self._flags = 0
        for k, v in util.get_iter(self._regex_flags):
            self._flags |= v if kwargs.pop(k, False) else 0

        super(StringMatches, self).__init__(value_type, kwargs)

    def _is_valid(self, value):
        if self.matches is not None:
            regex = re.compile(self.matches, self._flags)
            return regex.match(value)
        else:
            return True

    def _fail(self, value):
        return self.fail % (value)


class CharacterExclude(Constraint):
    keywords = {'exclude': str, 'ignore_case': bool}
    fail = '\'%s\' contains excluded character \'%s\''

    def _is_valid(self, value):
        # Check if the function has only been called due to ignore_case
        if self.exclude is not None:
            for char in self.exclude:
                if self.ignore_case is not None:
                    if not self.ignore_case:
                        if char in value:
                            self._failed_char = char
                            return False
                    else:
                        if char.casefold() in value.casefold():
                            self._failed_char = char
                            return False
                else:
                    if char in value:
                        self._failed_char = char
                        return False
            return True
        else:
            return True

    def _fail(self, value):
        return self.fail % (value, self._failed_char)


class IpVersion(Constraint):
    keywords = {'version': int}
    fail = 'IP version of %s is not %s'

    def _is_valid(self, value):
        try:
            ip = ipaddress.ip_interface(to_unicode(value))
        except ValueError:
            return False
        return self.version == ip.version

    def _fail(self, value):
        return self.fail % (value, self.version)



from pytest import raises

from .. import parser as par
from yamale.validators.validators import (
    Validator, String, Regex, Number, Integer, Boolean, List, Day, Timestamp,
    Ip, Mac)


def test_eval():
    assert eval('String()') == String()


def test_types():
    assert par.parse('String()') == String()
    assert par.parse('str()') == String()
    assert par.parse('regex()') == Regex()
    assert par.parse('num()') == Number()
    assert par.parse('int()') == Integer()
    assert par.parse('day()') == Day()
    assert par.parse('timestamp()') == Timestamp()
    assert par.parse('bool()') == Boolean()
    assert par.parse('list(str())') == List(String())
    assert par.parse('ip()') == Ip()
    assert par.parse('mac()') == Mac()


def test_custom_type():

    class my_validator(Validator):
        pass

    assert par.parse('custom()', {'custom': my_validator}) == my_validator()


def test_required():
    assert par.parse('str(required=True)').is_required
    assert par.parse('str(required=False)').is_optional


def test_syntax_error():
    with raises(SyntaxError):
        par.parse('eval()')



class DataPath(object):

    def __init__(self, *path):
        self._path = path

    def __add__(self, other):
        dp = DataPath()
        dp._path = self._path + other._path
        return dp

    def __str__(self):
        return '.'.join(map(str, (self._path)))

    def __repr__(self):
        return 'DataPath({})'.format(repr(self._path))



#!/usr/bin/env python
from setuptools import setup, find_packages

readme = open('README.md', encoding='utf-8').read()
license = open('LICENSE', encoding='utf-8').read()
version = open('yamale/VERSION', encoding='utf-8').read().strip()

setup(
    name='yamale',
    version=version,
    url='https://github.com/23andMe/Yamale',
    author='Bo Lopker',
    author_email='blopker@23andme.com',
    description='A schema and validator for YAML.',
    long_description=readme,
    long_description_content_type='text/markdown',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['pyyaml'],
    python_requires='>=3.8',
    entry_points={
        'console_scripts': ['yamale=yamale.command_line:main'],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ]
)
